@@ -197,12 +197,11 @@ class PageEditor(Page):
 
         # Emit http_headers after checks (send_page)
         request.disableHttpCaching(level=2)
-        request.emit_http_headers()
 
         # check if we want to load a draft
         use_draft = None
         if 'button_load_draft' in form:
-            wanted_draft_timestamp = int(form.get('draft_ts', ['0'])[0])
+            wanted_draft_timestamp = int(form.get('draft_ts', '0'))
             if wanted_draft_timestamp:
                 draft = self._load_draft()
                 if draft is not None:
@@ -214,7 +213,7 @@ class PageEditor(Page):
         if use_draft is not None:
             title = _('Draft of "%(pagename)s"')
             # Propagate original revision
-            rev = int(form['draft_rev'][0])
+            rev = int(form['draft_rev'])
             self.set_raw_body(use_draft, modified=1)
             preview = use_draft
         elif preview is None:
@@ -234,7 +233,7 @@ class PageEditor(Page):
 
         # get request parameters
         try:
-            text_rows = int(form['rows'][0])
+            text_rows = int(form['rows'])
         except StandardError:
             text_rows = self.cfg.edit_rows
             if request.user.valid:
@@ -276,9 +275,9 @@ Please review the page and save then. Do not save this page as it is!""")
             # If the page exists, we get the text from the page.
             # TODO: maybe warn if template argument was ignored because the page exists?
             raw_body = self.get_raw_body()
-        elif 'template' in form:
+        elif 'template' in request.values:
             # If the page does not exist, we try to get the content from the template parameter.
-            template_page = wikiutil.unquoteWikiname(form['template'][0])
+            template_page = wikiutil.unquoteWikiname(request.values['template'])
             template_page_escaped = wikiutil.escape(template_page)
             if request.user.may.read(template_page):
                 raw_body = Page(request, template_page).get_raw_body()
@@ -324,6 +323,7 @@ Please review the page and save then. Do not save this page as it is!""")
                      'lock_secs': lock_secs,
                     }) or '',
             editor_mode=1,
+            allow_doubleclick=1,
         )
 
         request.write(request.formatter.startContent("content"))
@@ -333,10 +333,9 @@ Please review the page and save then. Do not save this page as it is!""")
             raw_body = _('Describe %s here.') % (self.page_name, )
 
         # send form
-        request.write('<form id="editor" method="post" action="%s/%s#preview" onSubmit="flgChange = false;">' % (
-            request.getScriptname(),
-            wikiutil.quoteWikinameURL(self.page_name),
-            ))
+        request.write('<form id="editor" method="post" action="%s#preview" onSubmit="flgChange = false;">' % (
+                request.href(self.page_name)
+        ))
 
         # yet another weird workaround for broken IE6 (it expands the text
         # editor area to the right after you begin to type...). IE sucks...
@@ -352,7 +351,7 @@ Please review the page and save then. Do not save this page as it is!""")
         request.write('<input type="hidden" name="ticket" value="%s">' % wikiutil.createTicket(request))
 
         # Save backto in a hidden input
-        backto = form.get('backto', [None])[0]
+        backto = request.values.get('backto')
         if backto:
             request.write(unicode(html.INPUT(type="hidden", name="backto", value=backto)))
 
@@ -411,7 +410,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
     document.write('<label for="chktrivialtop">%(label)s</label>');
     //-->
 </script> ''' % {
-                'checked': ('', 'checked')[form.get('trivial', ['0'])[0] == '1'],
+                'checked': ('', 'checked')[form.get('trivial', '0') == '1'],
                 'label': _("Trivial change"),
             })
 
@@ -423,6 +422,15 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 
         lang = self.pi.get('language', request.cfg.language_default)
 
+        if not text_rows:
+            # if no specific value is given for editor height, but 0, we
+            # compute the rows from the raw_body line count plus some
+            # extra rows for adding new text in the editor. Maybe this helps
+            # with the "double slider" usability issue, esp. for devices like
+            # the iphone where you can't operate both sliders.
+            current_rows = len(raw_body.split('\n'))
+            text_rows = max(10, int(current_rows * 1.5))
+
         request.write(
             u'''\
 <textarea id="editor-textarea" name="savetext" lang="%(lang)s" dir="%(dir)s" rows="%(rows)d" cols="80"
@@ -461,7 +469,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <label for="chktrivial">%(label)s</label>
 
 ''' % {
-                'checked': ('', 'checked')[form.get('trivial', ['0'])[0] == '1'],
+                'checked': ('', 'checked')[form.get('trivial', '0') == '1'],
                 'label': _("Trivial change"),
                 })
 
@@ -470,7 +478,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <input type="checkbox" name="rstrip" id="chkrstrip" value="1" %(checked)s>
 <label for="chkrstrip">%(label)s</label>
 ''' % {
-            'checked': ('', 'checked')[form.get('rstrip', ['0'])[0] == '1'],
+            'checked': ('', 'checked')[form.get('rstrip', '0') == '1'],
             'label': _('Remove trailing whitespace from each line')
             })
         request.write("</p>")
@@ -518,10 +526,10 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         self._save_draft(newtext, rev) # shall we really save a draft on CANCEL?
         self.lock.release()
 
-        backto = request.form.get('backto', [None])[0]
+        backto = request.values.get('backto')
         if backto:
             pg = Page(request, backto)
-            request.http_redirect(pg.url(request, relative=False))
+            request.http_redirect(pg.url(request))
         else:
             request.theme.add_msg(_('Edit was cancelled.'), "error")
             self.send_page()
@@ -593,7 +601,7 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
         if not (request.user.may.delete(self.page_name)
                 and request.user.may.write(newpagename)):
             msg = _('You are not allowed to rename this page!')
-            raise self.AccessDenied, msg
+            raise self.AccessDenied(msg)
 
         if not newpagename:
             return False, _("You can't rename to an empty pagename.")
@@ -676,7 +684,7 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
 
             # Remove cache entry (if exists)
             pg = Page(self.request, self.page_name)
-            key = self.request.form.get('key', ['text_html'])[0]
+            key = self.request.form.get('key', 'text_html') # XXX see cleanup code in deletePage
             caching.CacheEntry(self.request, pg, key, scope='item').remove()
             caching.CacheEntry(self.request, pg, "pagelinks", scope='item').remove()
 
@@ -700,7 +708,7 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
         if not (request.user.may.write(self.page_name)
                 and request.user.may.delete(self.page_name)):
             msg = _('You are not allowed to delete this page!')
-            raise self.AccessDenied, msg
+            raise self.AccessDenied(msg)
 
         try:
             msg = self.saveText(u"deleted\n", 0, comment=comment or u'', deleted=True, notify=False)
@@ -773,6 +781,7 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
         signature = u.signature()
         variables = {
             'PAGE': self.page_name,
+            'TIMESTAMP': now,
             'TIME': "<<DateTime(%s)>>" % now,
             'DATE': "<<Date(%s)>>" % now,
             'ME': u.name,
@@ -788,8 +797,8 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
             # Users can define their own variables via
             # UserHomepage/MyDict, which override the default variables.
             userDictPage = u.name + "/MyDict"
-            if request.dicts.has_dict(userDictPage):
-                variables.update(request.dicts.dict(userDictPage))
+            if userDictPage in request.dicts:
+                variables.update(request.dicts[userDictPage])
 
         for name in variables:
             text = text.replace('@%s@' % name, variables[name])
@@ -950,10 +959,10 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
                     if err.errno == 2: # there was no 'current' file
                         time.sleep(0.1)
                     else:
-                        raise self.CouldNotLock, _("Page could not get locked. Unexpected error (errno=%d).") % err.errno
+                        raise self.CouldNotLock(_("Page could not get locked. Unexpected error (errno=%d).") % err.errno)
 
             if not got_lock:
-                raise self.CouldNotLock, _("Page could not get locked. Missing 'current' file?")
+                raise self.CouldNotLock(_("Page could not get locked. Missing 'current' file?"))
 
             # increment rev number of current(-locked) page
             f = file(clfn)
@@ -962,7 +971,7 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
             try:
                 rev = int(revstr)
             except ValueError, err:
-                raise self.SaveError, _("Unable to determine current page revision from the 'current' file. The page %s is damaged and cannot be edited right now.") % self.page_name
+                raise self.SaveError(_("Unable to determine current page revision from the 'current' file. The page %s is damaged and cannot be edited right now.") % self.page_name)
 
             if not was_deprecated:
                 if self.do_revision_backup or rev == 0:
@@ -980,9 +989,9 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
                     pass # we don't care for errors in the os.remove
                 # throw a nicer exception
                 if err.errno == errno.ENOSPC:
-                    raise self.SaveError, _("Cannot save page %s, no storage space left.") % self.page_name
+                    raise self.SaveError(_("Cannot save page %s, no storage space left.") % self.page_name)
                 else:
-                    raise self.SaveError, _("An I/O error occurred while saving page %s (errno=%d)") % (self.page_name, err.errno)
+                    raise self.SaveError(_("An I/O error occurred while saving page %s (errno=%d)") % (self.page_name, err.errno))
             # atomically put it in place (except on windows)
             else:
                 filesys.rename(cltfn, clfn)
@@ -1056,13 +1065,13 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
         msg = ""
         if not request.user.may.save(self, newtext, rev, **kw):
             msg = _('You are not allowed to edit this page!')
-            raise self.AccessDenied, msg
+            raise self.AccessDenied(msg)
         elif not self.isWritable():
             msg = _('Page is immutable!')
-            raise self.Immutable, msg
+            raise self.Immutable(msg)
         elif not newtext:
             msg = _('You cannot save empty pages.')
-            raise self.EmptyPage, msg
+            raise self.EmptyPage(msg)
         elif rev != 0 and rev != self.current_rev():
             # check if we already saved that page
             other = False
@@ -1081,16 +1090,16 @@ Try a different name.""", wiki=True) % (wikiutil.escape(newpagename), )
                     return msg
                 else:
                     msg = _("You already edited this page! Please do not use the back button.")
-                    raise self.EditConflict, msg
+                    raise self.EditConflict(msg)
 
                 msg = _("""Someone else saved this page while you were editing!
 Please review the page and save then. Do not save this page as it is!""")
 
-            raise self.EditConflict, msg
+            raise self.EditConflict(msg)
         elif newtext == self.get_raw_body():
             msg = _('You did not change the page content, not saved!')
             self.lock.release()
-            raise self.Unchanged, msg
+            raise self.Unchanged(msg)
         else:
             from MoinMoin.security import parseACL
             # Get current ACL and compare to new ACL from newtext. If
@@ -1102,7 +1111,7 @@ Please review the page and save then. Do not save this page as it is!""")
                 parseACL(request, newtext).acl != acl.acl and
                 action != "SAVE/REVERT"):
                 msg = _("You can't change ACLs on this page since you have no admin rights on it!")
-                raise self.NoAdmin, msg
+                raise self.NoAdmin(msg)
 
         presave = PagePreSaveEvent(request, self, newtext)
         results = send_event(presave)
@@ -1126,7 +1135,6 @@ Please review the page and save then. Do not save this page as it is!""")
             trivial = kw.get('trivial', 0)
             # write the page file
             mtime_usecs, rev = self._write_file(newtext, action, comment, extra, deleted=deleted)
-            self.clean_acl_cache()
             self._save_draft(None, None) # everything fine, kill the draft for this page
 
             if notify:
@@ -1241,14 +1249,14 @@ class PageLock:
         else:
             mins_valid = (secs_valid+59) / 60
             if self.locktype == 'lock':
-                # lout out user
+                # lock out user
+                timestamp_until = self.request.user.getFormattedDateTime(self.timestamp+secs_valid)
                 result = 0, _(
                     "This page is currently ''locked'' for editing by %(owner)s until %(timestamp)s,"
                     " i.e. for %(mins_valid)d minute(s).",
-                    wiki=True) % {'owner': owner, 'timestamp': timestamp, 'mins_valid': mins_valid}
+                    wiki=True) % {'owner': owner, 'timestamp': timestamp_until, 'mins_valid': mins_valid}
             else:
                 # warn user about existing lock
-
                 result = 1, _(
 """This page was opened for editing or last previewed at %(timestamp)s by %(owner)s.<<BR>>
 '''You should ''refrain from editing'' this page for at least another %(mins_valid)d minute(s),
