@@ -49,10 +49,10 @@ class revert(ActionBase):
     def do_action(self):
         """ revert pagename """
         form = self.form
-        comment = form.get('comment', [u''])[0]
+        comment = form.get('comment', u'')
         comment = wikiutil.clean_input(comment)
 
-        if self.request.request_method != 'POST':
+        if self.request.method != 'POST':
             return False, u''
 
         rev = self.request.rev
@@ -62,9 +62,7 @@ class revert(ActionBase):
             msg = pg.revertPage(rev, comment)
             # make it show the current version...
             self.request.rev = None
-        except PageEditor.RevertError, error:
-            msg = unicode(error)
-        except PageEditor.Unchanged, error:
+        except PageEditor.SaveError, error:
             msg = unicode(error)
 
         return True, msg
