@@ -31,6 +31,7 @@ from MoinMoin import log
 logging = log.getLogger(__name__)
 
 from MoinMoin import caching
+from MoinMoin.i18n import strings
 
 # This is a global for a reason: in persistent environments all languages in
 # use will be cached; Note: you have to restart if you update language data.
@@ -39,6 +40,11 @@ from MoinMoin import caching
 # this gets loaded early and completely:
 languages = None
 
+# system_pages has a dictionary containing all english
+# system page names and also all translated system pages names as keys,
+# see also wikiutil.isSystemPage:
+system_pages = {}
+
 translations = {}
 
 def po_filename(request, language, domain, i18n_dir='i18n'):
@@ -68,6 +74,9 @@ def i18n_init(request):
         if meta_cache.needsUpdate(i18n_dir):
             logging.debug("cache needs update")
             _languages = {}
+            _system_pages = {}
+            for pagename in strings.all_pages:
+                _system_pages[pagename] = ('en', pagename)
             for lang_file in glob.glob(po_filename(request, language='*', domain='MoinMoin')): # XXX only MoinMoin domain for now
                 language, domain, ext = os.path.basename(lang_file).split('.')
                 t = Translation(language, domain)
@@ -80,18 +89,28 @@ def i18n_init(request):
                 for key, value in t.info.items():
                     #logging.debug("meta key %s value %r" % (key, value))
                     _languages[language][key] = value.decode(encoding)
+                for pagename in strings.all_pages:
+                    try:
+                        pagename_translated = t.translation._catalog[pagename]
+                    except KeyError:
+                        pass
+                    else:
+                        _system_pages[pagename_translated] = (language, pagename)
             logging.debug("dumping language metadata to disk cache")
             try:
-                meta_cache.update(_languages)
+                meta_cache.update({
+                    'languages': _languages,
+                    'system_pages': _system_pages,
+                })
             except caching.CacheError:
                 pass
 
         if languages is None: # another thread maybe has done it before us
             try:
                 logging.debug("loading language metadata from disk cache")
-                _languages = meta_cache.content()
+                d = meta_cache.content()
                 if languages is None:
-                    languages = _languages
+                    globals().update(d)
             except caching.CacheError:
                 pass
     request.clock.stop('i18n_init')
@@ -270,7 +289,10 @@ def getText(original, request, lang, **kw):
     if original in translation.raw:
         translated = translation.raw[original]
         if formatted:
-            key = (original, percent)
+            # it is important to include siteid and percent into the key because
+            # formatted output depends on the (farm) wiki in which the page is
+            # rendered (e.g. for link urls) and also on the percent param
+            key = (original, request.cfg.siteid, percent)
             if key in translation.formatted:
                 translated = translation.formatted[key]
                 if translated is None:
@@ -288,8 +310,8 @@ def getText(original, request, lang, **kw):
             language = languages[lang]['x-language-in-english']
             dictpagename = "%sDict" % language.replace(' ', '')
             dicts = request.dicts
-            if dicts.has_dict(dictpagename):
-                userdict = dicts.dict(dictpagename)
+            if dictpagename in dicts:
+                userdict = dicts[dictpagename]
                 translated = userdict[original]
             else:
                 raise KeyError
@@ -365,15 +387,12 @@ def browserLanguages(request):
     the request, normalizing to lower case.
     """
     fallback = []
-    accepted = request.http_accept_language
+    accepted = request.accept_languages
     if accepted:
-        # Extract the languages names from the string
-        accepted = accepted.split(',')
-        accepted = [lang.split(';')[0] for lang in accepted]
         # Add base language for each sub language. If the user specified
         # a sub language like "en-us", we will try to to provide it or
         # a least the base language "en" in this case.
-        for lang in accepted:
+        for lang, quality in accepted:
             lang = lang.lower()
             fallback.append(lang)
             if '-' in lang:
