@@ -63,7 +63,7 @@ except ImportError:
             self.raw = raw
             self.request = request
 
-        def format(self, formatter):
+        def format(self, formatter, **kw):
             _ = self.request.getText
             from MoinMoin.parser.text import Parser as TextParser
             self.request.write(formatter.sysmsg(1) +
@@ -152,7 +152,7 @@ class MoinWriter(html4css1.Writer):
         if current_version < required_version:
             err = 'ERROR: The installed docutils version is %s;' % ('.'.join([str(i) for i in current_version]))
             err += ' version %s or later is required.' % ('.'.join([str(i) for i in required_version]))
-            raise RuntimeError, err
+            raise RuntimeError(err)
 
     def translate(self):
         visitor = MoinTranslator(self.document,
@@ -200,7 +200,7 @@ Lists: * bullets; 1., a. numbered items.
         self.request = request
         self.form = request.form
 
-    def format(self, formatter):
+    def format(self, formatter, **kw):
         # Create our simple parser
         parser = MoinDirectives(self.request)
 
@@ -391,6 +391,9 @@ class MoinTranslator(html4css1.HTMLTranslator):
                 # for images with targets).
                 if not [i for i in node.children if i.__class__ == docutils.nodes.image]:
                     node['classes'].append('interwiki')
+            elif prefix == 'javascript':
+                # is someone trying to do XSS with javascript?
+                node['refuri'] = 'javascript:alert("it does not work")'
             elif prefix != '':
                 # Some link scheme (http, file, https, mailto, etc.), add class
                 # information if the reference doesn't have a child image (don't
